﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace JobAssignmentProblem
{
    public class Program
    {
        public static void Main(string[] args)
        {
            int[,] matrix = new int[,]
            {
                { 11, 7, 10, 17, 10 },
                { 13, 21, 7, 11, 13 },
                { 13, 13, 15, 13,14 },
                { 18, 10, 13, 16,14 },
                { 12, 8, 16, 19, 10 }
            };
            AssignGreedy(matrix);
            Console.ReadLine();
        }

        public static void AssignGreedy(int[,] matrix)
        {
            int[,] reducedMatrix = new int[matrix.GetLength(0), matrix.GetLength(1)];

            // From each row, find the smallest number.
            // Extract that number from each cell on that row.
            // From Matrix to LossMatrix.
            for (int row = 0; row < matrix.GetLength(0); row++)
            {
                int[] wholeRow = Enumerable.Range(0, matrix.GetLength(0))
                                           .Select(column => matrix[row, column])
                                           .ToArray();
                int smallest = wholeRow.Min();
                for (int column = 0; column < matrix.GetLength(1); column++)
                {
                    reducedMatrix[row, column] = matrix[row, column] - smallest;
                }
            }

            // From each column, find the smallest number.
            // Extract that number from each cell on that column.
            // From LossMatrix to LossMatrix.
            for (int column = 0; column < matrix.GetLength(1); column++)
            {
                int[] wholeColumn = Enumerable.Range(0, reducedMatrix.GetLength(0))
                                              .Select(row => reducedMatrix[row, column])
                                              .ToArray();
                int smallest = wholeColumn.Min();
                for (int row = 0; row < reducedMatrix.GetLength(0); row++)
                {
                    reducedMatrix[row, column] = reducedMatrix[row, column] - smallest;
                }
            }

            // Find the zeroes in each row, save their corresponding columns.
            Dictionary<int, List<int>> zeroesInRows = new Dictionary<int, List<int>>();
            for (int row = 0; row < matrix.GetLength(0); row++)
            {
                zeroesInRows.Add(row, new List<int>());
                for (int col = 0; col < matrix.GetLength(1); col++)
                {
                    if (reducedMatrix[row, col] == 0)
                    {
                        zeroesInRows[row].Add(col);
                    }
                }
            }
            // Sort the rows by counts of zeros in ascending order.
            zeroesInRows = zeroesInRows.OrderBy(kvp => kvp.Value.Count).ToDictionary(kvp => kvp.Key, kvp => kvp.Value);

            //Dictionary<int, List<int>> zeroesInCols = new Dictionary<int, List<int>>();
            //for (int col = 0; col < matrix.GetLength(1); col++)
            //{
            //    zeroesInCols.Add(col, new List<int>());
            //    for (int row = 0; row < matrix.GetLength(0); row++)
            //    {
            //        if (reducedMatrix[row, col] == 0)
            //        {
            //            zeroesInCols[col].Add(row);
            //        }
            //    }
            //}
            //zeroesInCols = zeroesInCols.OrderBy(kvp => kvp.Value.Count).ToDictionary(kvp => kvp.Key, kvp => kvp.Value);

            List<int> skipCols = new List<int>();
            for (int row = 0; row < reducedMatrix.GetLength(0); row++)
            {
                zeroesInRows[row] = zeroesInRows[row].Except(skipCols).ToList();

                if (zeroesInRows[row].Count > 0)
                {
                    int col = zeroesInRows[row].First();
                    Console.WriteLine($"Worker {row} to Job {col}");
                    skipCols.Add(col);
                }
                else
                {
                    int minVal = int.MaxValue;
                    int minCol = 0;
                    foreach (var col in Enumerable.Range(0, reducedMatrix.GetLength(1)).Except(skipCols))
                    {
                        if (reducedMatrix[row, col] <= minVal)
                        {
                            minVal = reducedMatrix[row, col];
                            minCol = col;
                        }
                    }

                    Console.WriteLine($"Worker {row} to Job {minCol}");
                    skipCols.Add(minCol);
                }
            }
        }
    }
}